/**
 * @since 1.0.0
 */
import type { DynamoDBClientConfig } from "@aws-sdk/client-dynamodb";
import type { BatchExecuteStatementCommandInput, BatchExecuteStatementCommandOutput, BatchGetCommandInput, BatchGetCommandOutput, BatchWriteCommandInput, BatchWriteCommandOutput, DeleteCommandInput, DeleteCommandOutput, DynamoDBDocumentClient, ExecuteStatementCommandInput, ExecuteStatementCommandOutput, ExecuteTransactionCommandInput, ExecuteTransactionCommandOutput, GetCommandInput, GetCommandOutput, PutCommandInput, PutCommandOutput, QueryCommandInput, QueryCommandOutput, ScanCommandInput, ScanCommandOutput, TransactGetCommandInput, TransactGetCommandOutput, TransactWriteCommandInput, TransactWriteCommandOutput, TranslateConfig, UpdateCommandInput, UpdateCommandOutput } from "@aws-sdk/lib-dynamodb";
import type { ConditionalCheckFailedError, DuplicateItemError, IdempotentParameterMismatchError, InternalServerError, InvalidEndpointError, ItemCollectionSizeLimitExceededError, ProvisionedThroughputExceededError, RequestLimitExceededError, ResourceNotFoundError, SdkError, TransactionCanceledError, TransactionConflictError, TransactionInProgressError } from "@effect-aws/client-dynamodb";
import { type HttpHandlerOptions } from "@effect-aws/commons";
import type { Cause } from "effect";
import { Effect, Layer } from "effect";
import * as Instance from "./DynamoDBDocumentClientInstance.js";
interface DynamoDBDocumentService$ {
    readonly _: unique symbol;
    /**
     * @see {@link BatchExecuteStatementCommand}
     */
    batchExecuteStatement(args: BatchExecuteStatementCommandInput, options?: HttpHandlerOptions): Effect.Effect<BatchExecuteStatementCommandOutput, Cause.TimeoutException | SdkError | InternalServerError | RequestLimitExceededError>;
    /**
     * @see {@link BatchGetCommand}
     */
    batchGet(args: BatchGetCommandInput, options?: HttpHandlerOptions): Effect.Effect<BatchGetCommandOutput, Cause.TimeoutException | SdkError | InternalServerError | InvalidEndpointError | ProvisionedThroughputExceededError | RequestLimitExceededError | ResourceNotFoundError>;
    /**
     * @see {@link BatchWriteCommand}
     */
    batchWrite(args: BatchWriteCommandInput, options?: HttpHandlerOptions): Effect.Effect<BatchWriteCommandOutput, Cause.TimeoutException | SdkError | InternalServerError | InvalidEndpointError | ItemCollectionSizeLimitExceededError | ProvisionedThroughputExceededError | RequestLimitExceededError | ResourceNotFoundError>;
    /**
     * @see {@link DeleteCommand}
     */
    delete(args: DeleteCommandInput, options?: HttpHandlerOptions): Effect.Effect<DeleteCommandOutput, Cause.TimeoutException | SdkError | ConditionalCheckFailedError | InternalServerError | InvalidEndpointError | ItemCollectionSizeLimitExceededError | ProvisionedThroughputExceededError | RequestLimitExceededError | ResourceNotFoundError | TransactionConflictError>;
    /**
     * @see {@link ExecuteStatementCommand}
     */
    executeStatement(args: ExecuteStatementCommandInput, options?: HttpHandlerOptions): Effect.Effect<ExecuteStatementCommandOutput, Cause.TimeoutException | SdkError | ConditionalCheckFailedError | DuplicateItemError | InternalServerError | ItemCollectionSizeLimitExceededError | ProvisionedThroughputExceededError | RequestLimitExceededError | ResourceNotFoundError | TransactionConflictError>;
    /**
     * @see {@link ExecuteTransactionCommand}
     */
    executeTransaction(args: ExecuteTransactionCommandInput, options?: HttpHandlerOptions): Effect.Effect<ExecuteTransactionCommandOutput, Cause.TimeoutException | SdkError | IdempotentParameterMismatchError | InternalServerError | ProvisionedThroughputExceededError | RequestLimitExceededError | ResourceNotFoundError | TransactionCanceledError | TransactionInProgressError>;
    /**
     * @see {@link GetCommand}
     */
    get(args: GetCommandInput, options?: HttpHandlerOptions): Effect.Effect<GetCommandOutput, Cause.TimeoutException | SdkError | InternalServerError | InvalidEndpointError | ProvisionedThroughputExceededError | RequestLimitExceededError | ResourceNotFoundError>;
    /**
     * @see {@link PutCommand}
     */
    put(args: PutCommandInput, options?: HttpHandlerOptions): Effect.Effect<PutCommandOutput, Cause.TimeoutException | SdkError | ConditionalCheckFailedError | InternalServerError | InvalidEndpointError | ItemCollectionSizeLimitExceededError | ProvisionedThroughputExceededError | RequestLimitExceededError | ResourceNotFoundError | TransactionConflictError>;
    /**
     * @see {@link QueryCommand}
     */
    query(args: QueryCommandInput, options?: HttpHandlerOptions): Effect.Effect<QueryCommandOutput, Cause.TimeoutException | SdkError | InternalServerError | InvalidEndpointError | ProvisionedThroughputExceededError | RequestLimitExceededError | ResourceNotFoundError>;
    /**
     * @see {@link ScanCommand}
     */
    scan(args: ScanCommandInput, options?: HttpHandlerOptions): Effect.Effect<ScanCommandOutput, Cause.TimeoutException | SdkError | InternalServerError | InvalidEndpointError | ProvisionedThroughputExceededError | RequestLimitExceededError | ResourceNotFoundError>;
    /**
     * @see {@link TransactGetCommand}
     */
    transactGet(args: TransactGetCommandInput, options?: HttpHandlerOptions): Effect.Effect<TransactGetCommandOutput, Cause.TimeoutException | SdkError | InternalServerError | InvalidEndpointError | ProvisionedThroughputExceededError | RequestLimitExceededError | ResourceNotFoundError | TransactionCanceledError>;
    /**
     * @see {@link TransactWriteCommand}
     */
    transactWrite(args: TransactWriteCommandInput, options?: HttpHandlerOptions): Effect.Effect<TransactWriteCommandOutput, Cause.TimeoutException | SdkError | IdempotentParameterMismatchError | InternalServerError | InvalidEndpointError | ProvisionedThroughputExceededError | RequestLimitExceededError | ResourceNotFoundError | TransactionCanceledError | TransactionInProgressError>;
    /**
     * @see {@link UpdateCommand}
     */
    update(args: UpdateCommandInput, options?: HttpHandlerOptions): Effect.Effect<UpdateCommandOutput, Cause.TimeoutException | SdkError | ConditionalCheckFailedError | InternalServerError | InvalidEndpointError | ItemCollectionSizeLimitExceededError | ProvisionedThroughputExceededError | RequestLimitExceededError | ResourceNotFoundError | TransactionConflictError>;
}
/**
 * @since 1.0.0
 * @category constructors
 */
export declare const makeDynamoDBDocumentService: Effect.Effect<DynamoDBDocumentService$, never, Instance.DynamoDBDocumentClientInstance>;
declare const DynamoDBDocumentService_base: import("effect/Context").TagClass<DynamoDBDocumentService, "@effect-aws/lib-dynamodb/DynamoDBDocumentService", DynamoDBDocumentService$> & Effect.Tag.Proxy<DynamoDBDocumentService, DynamoDBDocumentService$> & {
    use: <X>(body: (_: DynamoDBDocumentService$) => X) => [X] extends [Effect.Effect<infer A, infer E, infer R>] ? Effect.Effect<A, E, DynamoDBDocumentService | R> : [X] extends [PromiseLike<infer A_1>] ? Effect.Effect<A_1, Cause.UnknownException, DynamoDBDocumentService> : Effect.Effect<X, never, DynamoDBDocumentService>;
};
/**
 * @since 1.0.0
 * @category models
 */
export declare class DynamoDBDocumentService extends DynamoDBDocumentService_base {
    static readonly defaultLayer: Layer.Layer<DynamoDBDocumentService, never, never>;
    static readonly layer: (config: DynamoDBDocumentService.Config) => Layer.Layer<DynamoDBDocumentService, never, never>;
    static readonly baseLayer: (evaluate: (defaultConfig: DynamoDBClientConfig) => DynamoDBDocumentClient) => Layer.Layer<DynamoDBDocumentService, never, never>;
}
/**
 * @since 1.0.0
 */
export declare namespace DynamoDBDocumentService {
    /**
     * @since 1.0.0
     */
    type Config = TranslateConfig;
}
export {};
//# sourceMappingURL=DynamoDBDocumentService.d.ts.map